@@ -449,12 +449,14 @@ inline Code *code_tags(code_alc_t &alc, const std::string &fmt, const std::strin
 {
     Code *x = new_code(alc, mtags ? Code::MTAGS : Code::STAGS);
 
-    char *fmt_copy = alc.alloct<char>(fmt.length() + 1);
-    strcpy(fmt_copy, fmt.c_str());
+    const size_t fmt_size = fmt.length() + 1;
+    char *fmt_copy = alc.alloct<char>(fmt_size);
+    memcpy(fmt_copy, fmt.c_str(), fmt_size);
     x->tags.fmt = fmt_copy;
 
-    char *sep_copy = alc.alloct<char>(sep.length() + 1);
-    strcpy(sep_copy, sep.c_str());
+    const size_t sep_size = sep.length() + 1;
+    char *sep_copy = alc.alloct<char>(sep_size);
+    memcpy(sep_copy, sep.c_str(), sep_size);
     x->tags.sep = sep_copy;
 
     return x;
@@ -783,14 +785,13 @@ void gen_goto(Output &output, const DFA &dfa, CodeList *stmts, const State *from
     const CodeJump &jump);
 const char *gen_lessthan(Scratchbuf &o, const opt_t *opts, size_t n);
 void gen_code(Output &output, dfas_t &dfas);
-const char *gen_eof_label(Output &output, const DFA &dfa);
 
 void expand(CodegenContext &ctx, Code *code);
 void combine(CodegenContext &ctx, Code *code);
 void render(RenderContext &rctx, const Code *code);
 
 bool consume(const State *s);
-std::string tag_expr(const Tag &tag, bool lvalue);
+void expand_fintags(const Tag &tag, std::vector<std::string> &fintags);
 std::string vartag_name(tagver_t ver, const std::string &prefix);
 std::string vartag_expr(tagver_t ver, const opt_t *opts);
 void output_version_time(std::ostream &os, const opt_t *opts);
