@@ -33,18 +33,6 @@ static void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa,
     const Rule &rule);
 static bool endstate(const State *s);
 
-const char *gen_eof_label(Output &output, const DFA &dfa)
-{
-    const opt_t *opts = output.block().opts;
-    Scratchbuf &o = output.scratchbuf;
-    DASSERT(o.empty());
-
-    // Include block ID and condition name to avoid duplicate EOF labels.
-    o.str(opts->labelPrefix).cstr("eof").u64(output.blockid());
-    if (!dfa.cond.empty()) o.cstr("_").str(dfa.cond);
-    return o.flush();
-}
-
 static const char *gen_fill_label(Output &output, uint32_t index)
 {
     const opt_t *opts = output.block().opts;
@@ -312,38 +300,44 @@ static CodeList *gen_fill_falllback(Output &output, const DFA &dfa,
 {
     const opt_t *opts = output.block().opts;
     code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    const char *text;
+    const State *fallback;
+    tcid_t falltags;
 
-    const bool final = from->rule != Rule::NONE;
-    const State *fallback = final ? dfa.finstates[from->rule] : dfa.defstate;
-    tcid_t falltags = final ? from->rule_tags : from->fall_tags;
+    DASSERT(opts->eof != NOEOF);
+    if (from->action.type == Action::INITIAL) {
+        // EOF in the initial state: EOF rule takes priority over any other rule.
+        fallback = dfa.eof_state;
+        falltags = TCID0;
+    } else if (from->rule != Rule::NONE) {
+        // EOF in accepting state: match the rule in this state.
+        fallback = dfa.finstates[from->rule];
+        falltags = from->rule_tags;
+    } else {
+        // EOF in a non-accepting state: fallback to default state.
+        fallback = dfa.defstate;
+        falltags = from->fall_tags;
+    }
 
-    // If tags have been hoisted, do not re-add them to fallback transition.
-    if (from->go.tags != TCID0) {
+    if (opts->stadfa) {
+        DASSERT(from->go.tags == from->stadfa_tags);
+
+    } else if (from->go.tags != TCID0) {
+        // Tags have been hoisted out of transitions into state (this means that
+        // tags on all transitions coincide, including the fallback transition).
+        // Do not add duplicate tags to fallback transition.
         DASSERT(from->go.tags == falltags);
         falltags = TCID0;
     }
 
     CodeList *fallback_trans = code_list(alc);
-    if (from->action.type == Action::INITIAL) {
-        // initial state: if accepting, go to eof state, else go to default state
-        if (final) {
-            fallback->label->used = true;
-            text = o.str(opts->labelPrefix).label(*fallback->label).flush();
-        } else {
-            text = gen_eof_label(output, dfa);
-        }
-        append(fallback_trans, code_stmt(alc, o.cstr("goto ").cstr(text).flush()));
-    }
-    else if (fallback != to || opts->fFlag) {
+    if (fallback != to || opts->fFlag) {
         // tag actions on the fallback transition
         gen_settags(output, fallback_trans, dfa, falltags, false /* delayed */);
 
         // go to fallback state
         fallback->label->used = true;
-        text = o.cstr("goto ").str(opts->labelPrefix).label(*fallback->label).flush();
-        append(fallback_trans, code_stmt(alc, text));
+        append(fallback_trans, code_stmt(alc, output.scratchbuf.cstr("goto ")
+            .str(opts->labelPrefix).label(*fallback->label).flush()));
     }
     else {
         // Transition can be elided, because control flow "falls through" to an
@@ -434,19 +428,29 @@ static void gen_fill(Output &output, CodeList *stmts, const DFA &dfa,
 void gen_fill_and_label(Output &output, CodeList *stmts, const DFA &dfa, const State *s)
 {
     const opt_t *opts = output.block().opts;
-    if (!opts->fill_use || endstate(s)) return;
 
-    const bool need_fill = opts->eof == NOEOF && s->fill > 0;
-    const bool need_fill_label = (need_fill && opts->fFlag) || opts->eof != NOEOF;
+    const bool need_fill = opts->fill_use && !endstate(s);
+    const bool need_fill_on_trans = need_fill && opts->eof != NOEOF;
+    const bool need_fill_in_state = need_fill && opts->eof == NOEOF && s->fill > 0;
+    const bool need_fill_label = need_fill_on_trans || (need_fill_in_state && opts->fFlag);
 
     if (need_fill_label) {
         ++output.block().fill_index;
     }
 
-    if (need_fill) {
+    if (need_fill_in_state) {
         gen_fill(output, stmts, dfa, s, NULL);
     }
 
+    if (opts->eof != NOEOF) {
+        // If the end-of-input rule $ is used, the lexer may jump to the YYFILL
+        // label to rescan the current input character. Generate tag operations
+        // before the label to avoid applying them multiple times in the above
+        // scenario (re-application may produce incorrect results in case of
+        // non-idempotent operations).
+        gen_settags(output, stmts, dfa, s->go.tags, opts->stadfa /* delayed */);
+    }
+
     if (need_fill_label) {
         const char *flabel = gen_fill_label(output, output.block().fill_index - 1);
         append(stmts, code_slabel(output.allocator, flabel));
@@ -539,11 +543,34 @@ static void gen_assign(Output &output, CodeList *stmts, const std::string &lhs,
     const std::string &rhs)
 {
     Scratchbuf &o = output.scratchbuf;
-
     o.str(lhs).cstr(" = ").str(rhs);
     append(stmts, code_stmt(output.allocator, o.flush()));
 }
 
+static void gen_assign_many(Output &output, CodeList *stmts,
+    const std::vector<std::string> &many, const std::string &rhs)
+{
+    Scratchbuf &o = output.scratchbuf;
+    for (size_t i = 0; i < many.size(); ++i) {
+        o.str(many[i]).cstr(" = ");
+    }
+    o.str(rhs);
+    append(stmts, code_stmt(output.allocator, o.flush()));
+}
+
+static void gen_assign_many_to_first(Output &output, CodeList *stmts,
+    const std::vector<std::string> &many)
+{
+    if (many.size() <= 1) return;
+
+    Scratchbuf &o = output.scratchbuf;
+    for (size_t i = 1; i < many.size(); ++i) {
+        o.str(many[i]).cstr(" = ");
+    }
+    o.str(many[0]);
+    append(stmts, code_stmt(output.allocator, o.flush()));
+}
+
 static void gen_restorectx(Output &output, CodeList *stmts, const std::string &tag)
 {
     const opt_t *opts = output.block().opts;
@@ -647,6 +674,7 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
     const tagver_t *fins = dfa.finvers;
     code_alc_t &alc = output.allocator;
     Scratchbuf &o = output.scratchbuf;
+    std::vector<std::string> fintags;
 
     if (rule.ncap > 0) {
         o.cstr("yynmatch = ").u64(rule.ncap);
@@ -656,12 +684,18 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
     CodeList *varops = code_list(alc);
     CodeList *fixops = code_list(alc);
     CodeList *fixtrailops = code_list(alc);
+    CodeList *fixpostops = code_list(alc);
+    std::string negtag;
 
     for (size_t t = rule.ltag; t < rule.htag; ++t) {
         const Tag &tag = tags[t];
-        if (fictive(tag)) {
-            // structural tag that is only needed for disambiguation
-        } else if (!fixed(tag)) {
+
+        // structural tag that is only needed for disambiguation
+        if (fictive(tag)) continue;
+
+        expand_fintags(tag, fintags);
+
+        if (!fixed(tag)) {
             // variable tag
             const std::string expr = vartag_expr(fins[t], opts);
             if (trailing(tag)) {
@@ -673,9 +707,11 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
                         notag ? opts->yyctxmarker : expr);
                 }
             } else {
-                gen_assign(output, varops, tag_expr(tag, true), expr);
+                gen_assign_many(output, varops, fintags, expr);
             }
         } else {
+            DASSERT(!history(tag));
+
             // fixed tag that is based on either variable tag or cursor
             const int32_t dist = static_cast<int32_t>(tag.dist);
             const bool fixed_on_cursor = tag.base == Tag::RIGHTMOST;
@@ -683,8 +719,8 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
                 ? opts->yycursor : vartag_expr(fins[tag.base], opts);
 
             if (trailing(tag)) {
+                DASSERT(tag.toplevel);
                 if (generic) {
-                    DASSERT(!history(tag));
                     if (!fixed_on_cursor) {
                         gen_restorectx(output, fixtrailops, base);
                     }
@@ -699,18 +735,50 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
                     append(fixtrailops, code_stmt(alc, o.flush()));
                 }
             } else {
-                const std::string fix = tag_expr(tag, true);
+                DASSERT(!fintags.empty());
+                const std::string &first = fintags[0];
+
                 if (generic) {
                     if (fixed_on_cursor) {
-                        gen_settag(output, fixops, fix, false, history(tag));
+                        gen_settag(output, fixops, first, false, false);
+                        gen_shift(output, fixops, -dist, first, false);
+                        gen_assign_many_to_first(output, fixops, fintags);
+                    } else if (dist == 0) {
+                        gen_assign_many(output, fixops, fintags, base);
+                    } else if (tag.toplevel) {
+                        gen_assign(output, fixops, first, base);
+                        gen_shift(output, fixops, -dist, first, false);
+                        gen_assign_many_to_first(output, fixops, fintags);
                     } else {
-                        gen_assign(output, fixops, fix, base);
+                        // Split operations in two parts. First, set all fixed tags to
+                        // their base tag. Second, choose one of the base tags to store
+                        // negative value (with generic API there is no NULL constant)
+                        // and compare fixed tags against it before shifting. This must
+                        // be done after all uses of that base tag.
+                        if (negtag.empty()) negtag = base;
+                        gen_assign(output, fixops, first, base);
+                        const char *cond = o.str(first).cstr(" != ").str(negtag).flush();
+                        CodeList *then = code_list(alc);
+                        gen_shift(output, then, -dist, first, false);
+                        append(fixpostops, code_if_then_else(alc, cond, then, NULL));
                     }
-                    gen_shift(output, fixops, -dist, fix, false);
                 } else {
-                    o.str(fix).cstr(" = ").str(base);
-                    if (dist > 0) o.cstr(" - ").i32(dist);
-                    append(fixops, code_stmt(alc, o.flush()));
+                    if (dist == 0) {
+                        gen_assign_many(output, fixops, fintags, base);
+                    } else if (tag.toplevel) {
+                        gen_assign_many(output, fixops, fintags,
+                            o.str(base).cstr(" - ").i32(dist).flush());
+                    } else {
+                        // If base tag is NULL, fixed tag is also NULL, otherwise it
+                        // equals the value of the base tag plus offset.
+                        gen_assign(output, fixops, first, base);
+                        const char *cond = o.str(base).cstr(" != NULL").flush();
+                        CodeList *then = code_list(alc);
+                        append(then, code_stmt(alc,
+                            o.str(first).cstr(" -= ").i32(dist).flush()));
+                        append(fixops, code_if_then_else(alc, cond, then, NULL));
+                        gen_assign_many_to_first(output, fixops, fintags);
+                    }
                 }
             }
         }
@@ -719,25 +787,31 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
     append(stmts, varops);
     append(stmts, fixtrailops);
     append(stmts, fixops);
+
+    if (!negtag.empty()) {
+        // With generic API there is no explicit negative NULL value, so it is
+        // necessary to materialize no-match value in a tag.
+        DASSERT(opts->input_api == INPUT_CUSTOM);
+        append(stmts, code_text(alc, o.cstr("/* materialize no-match value */").flush()));
+        gen_settag(output, stmts, negtag, true, false);
+        append(stmts, fixpostops);
+    }
 }
 
-std::string tag_expr(const Tag &tag, bool lvalue)
+void expand_fintags(const Tag &tag, std::vector<std::string> &fintags)
 {
-    DASSERT(!trailing(tag));
-
-    // named tag
-    if (!capture(tag)) {
-        return *tag.name;
-    }
-
-    // capture tag, maps to a range of parentheses
-    std::string s = "yypmatch[" + to_string(tag.lsub) + "]";
-    if (lvalue) {
-        for (size_t i = tag.lsub + 2; i <= tag.hsub; i += 2) {
-            s += " = yypmatch[" + to_string(i) + "]";
+    fintags.clear();
+    if (trailing(tag)) {
+        // empty list
+    } else if (!capture(tag)) {
+        // named tag
+        fintags.push_back(*tag.name);
+    } else {
+        // capture tag, maps to a range of parentheses
+        for (size_t i = tag.lsub; i <= tag.hsub; i += 2) {
+            fintags.push_back("yypmatch[" + to_string(i) + "]");
         }
     }
-    return s;
 }
 
 bool endstate(const State *s)
