@@ -23,7 +23,7 @@ static const uint32_t NOCLOS = ~0u;
 
 struct nfa_state_t
 {
-    enum type_t {ALT, RAN, TAG, FIN, NIL} type;
+    enum type_t {ALT, RAN, TAG, FIN} type;
     union
     {
         struct
@@ -41,10 +41,6 @@ struct nfa_state_t
             nfa_state_t *out;
             tag_info_t info;
         } tag;
-        struct
-        {
-            nfa_state_t *out;
-        } nil;
     };
     size_t rule;
 
@@ -93,12 +89,6 @@ struct nfa_state_t
         type = FIN;
         init(r);
     }
-    void make_nil(size_t r, nfa_state_t *s)
-    {
-        type = NIL;
-        nil.out = s;
-        init(r);
-    }
 };
 
 struct nfa_t
