@@ -59,6 +59,12 @@ enum api_style_t {
     API_FREEFORM
 };
 
+enum fixed_tags_t {
+    FIXTAG_NONE,
+    FIXTAG_TOPLEVEL,
+    FIXTAG_ALL
+};
+
 enum parse_opts_t {
     OK,
     EXIT_OK,
@@ -87,20 +93,22 @@ const uint32_t NOEOF = ~0u - 1;
     /* files */ \
     CONSTOPT (std::string, source_file, "") \
     CONSTOPT (std::string, output_file, "") \
+    CONSTOPT (std::string, dep_file, "") \
     /* include paths */ \
     CONSTOPT (std::vector<std::string>, incpaths, std::vector<std::string>()) \
     /* input encoding */ \
     CONSTOPT (Enc::type_t, input_encoding, Enc::ASCII) \
     /* internals */ \
-    CONSTOPT (bool, dfa, true) \
     CONSTOPT (dfa_minimization_t, dfa_minimization, DFA_MINIMIZATION_MOORE) \
     CONSTOPT (posix_closure_t, posix_closure, POSIX_CLOSURE_GOR1) \
     CONSTOPT (posix_prectable_alg_t, posix_prectable, POSIX_PRECTABLE_COMPLEX) \
+    CONSTOPT (fixed_tags_t, fixed_tags, FIXTAG_ALL) \
     CONSTOPT (bool, lookahead, true) \
     CONSTOPT (bool, eager_skip, false) \
     CONSTOPT (bool, optimize_tags, true) \
     CONSTOPT (bool, backward, false) \
     CONSTOPT (bool, stadfa, false) \
+    CONSTOPT (bool, nested_negative_tags, true) \
     /* debug */ \
     CONSTOPT (bool, dump_nfa, false) \
     CONSTOPT (bool, dump_dfa_raw, false) \
