@@ -7,6 +7,7 @@ namespace re2c {
 const size_t Tag::RIGHTMOST = std::numeric_limits<size_t>::max();
 const uint32_t Tag::VARDIST = std::numeric_limits<uint32_t>::max();
 const size_t Tag::FICTIVE = Tag::RIGHTMOST - 1;
+const size_t Tag::NONE = Tag::RIGHTMOST - 2;
 
 
 Tag::Tag(const std::string *nm, bool hi, int32_t ht)
@@ -19,6 +20,7 @@ Tag::Tag(const std::string *nm, bool hi, int32_t ht)
     , hnest(Tag::RIGHTMOST)
     , history(hi)
     , orbit(false)
+    , toplevel(false)
     , height(ht)
 {}
 
@@ -33,6 +35,7 @@ Tag::Tag(size_t lsub, size_t hsub, bool ob, int32_t ht)
     , hnest(Tag::RIGHTMOST)
     , history(false)
     , orbit(ob)
+    , toplevel(false)
     , height(ht)
 {}
 
