@@ -16,10 +16,9 @@ static const tagver_t TAGVER_BOTTOM = std::numeric_limits<tagver_t>::min(); // d
 static const tagver_t TAGVER_ZERO = 0; // absense of tag
 static const tagver_t TAGVER_CURSOR = std::numeric_limits<tagver_t>::max(); // current position, highest priority
 
-struct tag_info_t
-{
-    uint32_t idx : 31;
-    uint32_t neg : 1;
+struct tag_info_t {
+    uint32_t idx; // tag index (starting from zero)
+    uint32_t neg; // tag sign (1 for negative, 0 for positive)
 };
 
 struct Tag
@@ -27,6 +26,7 @@ struct Tag
     static const size_t RIGHTMOST;
     static const uint32_t VARDIST;
     static const size_t FICTIVE;
+    static const size_t NONE;
 
     const std::string *name;
     size_t lsub;
@@ -37,6 +37,7 @@ struct Tag
     size_t hnest;
     bool history;
     bool orbit;
+    bool toplevel;
     int32_t height;
 
     Tag(const std::string *nm, bool hi, int32_t ht);
@@ -45,10 +46,7 @@ struct Tag
 
 inline bool operator == (const tag_info_t &x, const tag_info_t &y)
 {
-    // per-component comparison is slower
-    RE2C_STATIC_ASSERT(sizeof(tag_info_t) == sizeof(uint32_t));
-    return *reinterpret_cast<const uint32_t*>(&x)
-        == *reinterpret_cast<const uint32_t*>(&y);
+    return x.idx == y.idx && x.neg == y.neg;
 }
 
 inline bool fixed(const Tag &tag)
